# Table of content

## Index

+ [Introduction](./introduction/StarRocks_intro.md)
+ Quick Start
  + [Concepts](./quick_start/Concepts.md)
  + [Architecture](./quick_start/Architecture.md)
  + [Deploy](./quick_start/Deploy.md)
  + [Data Flow and Control Flow](./quick_start/Data_flow_and_control_flow.md)
  + [Import and Query](./quick_start/Import_and_query.md)
  + [Test FAQs](./quick_start/Test_faq.md)
+ Table Design
  + [StarRocks Table Design](./table_design/StarRocks_table_design.md)
  + [Data Model](./table_design/Data_model.md)
  + [Data Distribution](./table_design/Data_distribution.md)
  + [Sort Key and Shortkey Index](./table_design/Sort_key.md)
  + [Materialized View](./table_design/Materialized_view.md)
  + [Bitmap Indexing](./table_design/Bitmap_index.md)
  + [Bloomfilter Indexing](./table_design/Bloomfilter_index.md)
+ Data Loading
  + [Loading Intro](./loading/Loading_intro.md)
  + [Stream Load](./loading/StreamLoad.md)
  + [Broker Load](./loading/BrokerLoad.md)
  + [Routine Load](./loading/RoutineLoad.md)
  + [Spark Load](./loading/SparkLoad.md)
  + [Insert Into](./loading/InsertInto.md)
  + [Load data into tables of Primary Key model](./loading/Load_to_Primary_Key_tables.md)
  + [ETL in Loading](./loading/Etl_in_loading.md)
  + [Json Loading](./loading/Json_loading.md)
  + [Synchronize data from MySQL](./loading/Flink_cdc_load.md)
  + [Load data by using flink-connector-starrocks](./loading/Flink-connector-starrocks.md)
  + [DataX Writer](./loading/DataX-starrocks-writer.md)
+ Data Export
  + [Export](./unloading/Export.md)
  + [Spark Connector](./unloading/Spark_connector.md)
  + [Flink Connector](./unloading/Flink_connector.md)
+ Using StarRocks
  + [Precise De-duplication with Bitmap](./using_starrocks/Using_bitmap.md)
  + [Approximate De-duplication with HLL](./using_starrocks/Using_HLL.md)
  + [Materialized View](./using_starrocks/Materialized_view.md)
  + [Colocate Join](./using_starrocks/Colocation_join.md)
  + [External Table](./using_starrocks/External_table.md)
  + [Array](./using_starrocks/Array.md)
  + [Window Function](./using_starrocks/Window_function.md)
  + [Java UDFs](./using_starrocks/Java-UDFs.md)
  + [Cost Based Optimizer](./using_starrocks/Cost_based_optimizer.md)
  + [Lateral Join](./using_starrocks/Lateral_join.md)
  + [Configure a time zone](./using_starrocks/timezone.md)
  + [File manager](./using_starrocks/filemanager.md)
+ Reference
  + SQL Reference
    + User Account Management
      + [ALTER USER](./sql-reference/sql-statements/account-management/ALTER%20USER.md)
      + [CREATE ROLE](./sql-reference/sql-statements/account-management/CREATE%20ROLE.md)
      + [CREATE USER](./sql-reference/sql-statements/account-management/CREATE%20USER.md)
      + [DROP ROLE](./sql-reference/sql-statements/account-management/DROP%20ROLE.md)
      + [DROP USER](./sql-reference/sql-statements/account-management/DROP%20USER.md)
      + [GRANT](./sql-reference/sql-statements/account-management/GRANT.md)
      + [REVOKE](./sql-reference/sql-statements/account-management/REVOKE.md)
      + [SET PASSWORD](./sql-reference/sql-statements/account-management/SET%20PASSWORD.md)
      + [SET PROPERTY](./sql-reference/sql-statements/account-management/SET%20PROPERTY.md)
      + [SHOW GRANTS](./sql-reference/sql-statements/account-management/SHOW%20GRANTS.md)
    + Cluster Management
      + [ADMIN CANCEL REPAIR](./sql-reference/sql-statements/Administration/ADMIN%20CANCEL%20REPAIR.md)
      + [ADMIN CHECK TABLET](./sql-reference/sql-statements/Administration/ADMIN%20CHECK%20TABLET.md)
      + [ADMIN REPAIR](./sql-reference/sql-statements/Administration/ADMIN%20REPAIR.md)
      + [ADMIN SET CONFIG](./sql-reference/sql-statements/Administration/ADMIN%20SET%20CONFIG.md)
      + [ADMIN SET REPLICA STATUS](./sql-reference/sql-statements/Administration/ADMIN%20SET%20REPLICA%20STATUS.md)
      + [ADMIN SHOW CONFIG](./sql-reference/sql-statements/Administration/ADMIN%20SHOW%20CONFIG.md)
      + [ADMIN SHOW REPLICA DISTRIBUTION](./sql-reference/sql-statements/Administration/ADMIN%20SHOW%20REPLICA%20DISTRIBUTION.md)
      + [ADMIN SHOW REPLICA STATUS](./sql-reference/sql-statements/Administration/ADMIN%20SHOW%20REPLICA%20STATUS.md)
      + [ALTER SYSTEM](./sql-reference/sql-statements/Administration/ALTER%20SYSTEM.md)
      + [CANCEL DECOMMISSION](./sql-reference/sql-statements/Administration/CANCEL%20DECOMMISSION.md)
      + [CREATE FILE](./sql-reference/sql-statements/Administration/CREATE%20FILE.md)
      + [DROP FILE](./sql-reference/sql-statements/Administration/DROP%20FILE.md)
      + [INSTALL PLUGIN](./sql-reference/sql-statements/Administration/INSTALL%20PLUGIN.md)
      + [SHOW BACKENDS](./sql-reference/sql-statements/Administration/SHOW%20BACKENDS.md)
      + [SHOW BROKER](./sql-reference/sql-statements/Administration/SHOW%20BROKER.md)
      + [SHOW FILE](./sql-reference/sql-statements/Administration/SHOW%20FILE.md)
      + [SHOW FRONTENDS](./sql-reference/sql-statements/Administration/SHOW%20FRONTENDS.md)
      + [SHOW FULL COLUMNS](./sql-reference/sql-statements/Administration/SHOW%20FULL%20COLUMNS.md)
      + [SHOW INDEX](./sql-reference/sql-statements/Administration/SHOW%20INDEX.md)
      + [SHOW PLUGINS](./sql-reference/sql-statements/Administration/SHOW%20PLUGINS.md)
      + [SHOW TABLE STATUS](./sql-reference/sql-statements/Administration/SHOW%20TABLE%20STATUS.md)
      + [UNINSTALL PLUGIN](./sql-reference/sql-statements/Administration/UNINSTALL%20PLUGIN.md)
    + DDL
      + [ALTER DATABASE](./sql-reference/sql-statements/data-definition/ALTER%20DATABASE.md)
      + [ALTER TABLE](./sql-reference/sql-statements/data-definition/ALTER%20TABLE.md)
      + [ALTER VIEW](./sql-reference/sql-statements/data-definition/ALTER%20VIEW.md)
      + [BACKUP](./sql-reference/sql-statements/data-definition/BACKUP.md)
      + [CANCEL BACKUP](./sql-reference/sql-statements/data-definition/CANCEL%20BACKUP.md)
      + [CANCEL RESTORE](./sql-reference/sql-statements/data-definition/CANCEL%20RESTORE.md)
      + [CREATE DATABASE](./sql-reference/sql-statements/data-definition/CREATE%20DATABASE.md)
      + [CREATE INDEX](./sql-reference/sql-statements/data-definition/CREATE%20INDEX.md)
      + [CREATE MATERIALIZED VIEW](./sql-reference/sql-statements/data-definition/CREATE%20MATERIALIZED%20VIEW.md)
      + [CREATE REPOSITORY](./sql-reference/sql-statements/data-definition/CREATE%20REPOSITORY.md)
      + [CREATE RESOURCE](./sql-reference/sql-statements/data-definition/CREATE%20RESOURCE.md)
      + [CREATE TABLE AS SELECT](./sql-reference/sql-statements/data-definition/CREATE%20TABLE%20AS%20SELECT.md)
      + [CREATE TABLE LIKE](./sql-reference/sql-statements/data-definition/CREATE%20TABLE%20LIKE.md)
      + [CREATE TABLE](./sql-reference/sql-statements/data-definition/CREATE%20TABLE.md)
      + [CREATE VIEW](./sql-reference/sql-statements/data-definition/CREATE%20VIEW.md)
      + [CREATE FUNCTION](./sql-reference/sql-statements/data-definition/create-function.md)
      + [DROP DATABASE](./sql-reference/sql-statements/data-definition/DROP%20DATABASE.md)
      + [DROP INDEX](./sql-reference/sql-statements/data-definition/DROP%20INDEX.md)
      + [DROP MATERIALIZED VIEW](./sql-reference/sql-statements/data-definition/DROP%20MATERIALIZED%20VIEW.md)
      + [DROP REPOSITORY](./sql-reference/sql-statements/data-definition/DROP%20REPOSITORY.md)
      + [DROP RESOURCE](./sql-reference/sql-statements/data-definition/DROP%20RESOURCE.md)
      + [DROP TABLE](./sql-reference/sql-statements/data-definition/DROP%20TABLE.md)
      + [DROP VIEW](./sql-reference/sql-statements/data-definition/DROP%20VIEW.md)
      + [DROP FUNCTION](./sql-reference/sql-statements/data-definition/drop-function.md)
      + [HLL](./sql-reference/sql-statements/data-definition/HLL.md)
      + [RECOVER](./sql-reference/sql-statements/data-definition/RECOVER.md)
      + [RESTORE](./sql-reference/sql-statements/data-definition/RESTORE.md)
      + [SHOW RESOURCES](./sql-reference/sql-statements/data-definition/SHOW%20RESOURCES.md)
      + [SHOW FUNCTION](./sql-reference/sql-statements/data-definition/show-functions.md)
      + [TRUNCATE TABLE](./sql-reference/sql-statements/data-definition/TRUNCATE%20TABLE.md)
    + DML
      + [ALTER ROUTINE LOAD](./sql-reference/sql-statements/data-manipulation/alter-routine-load.md)
      + [BROKER LOAD](./sql-reference/sql-statements/data-manipulation/BROKER%20LOAD.md)
      + [CANCEL LOAD](./sql-reference/sql-statements/data-manipulation/CANCEL%20LOAD.md)
      + [DELETE](./sql-reference/sql-statements/data-manipulation/DELETE.md)
      + [EXPORT](./sql-reference/sql-statements/data-manipulation/EXPORT.md)
      + [GROUP BY](./sql-reference/sql-statements/data-manipulation/GROUP%20BY.md)
      + [INSERT](./sql-reference/sql-statements/data-manipulation/insert.md)
      + [PAUSE ROUTINE LOAD](./sql-reference/sql-statements/data-manipulation/PAUSE%20ROUTINE%20LOAD.md)
      + [RESUME ROUTINE LOAD](./sql-reference/sql-statements/data-manipulation/RESUME%20ROUTINE%20LOAD.md)
      + [ROUTINE LOAD](./sql-reference/sql-statements/data-manipulation/ROUTINE%20LOAD.md)
      + [SELECT](./sql-reference/sql-statements/data-manipulation/SELECT.md)
      + [SHOW ALTER](./sql-reference/sql-statements/data-manipulation/SHOW%20ALTER.md)
      + [SHOW BACKUP](./sql-reference/sql-statements/data-manipulation/SHOW%20BACKUP.md)
      + [SHOW DATA](./sql-reference/sql-statements/data-manipulation/SHOW%20DATA.md)
      + [SHOW DATABASES](./sql-reference/sql-statements/data-manipulation/SHOW%20DATABASES.md)
      + [SHOW DELETE](./sql-reference/sql-statements/data-manipulation/SHOW%20DELETE.md)
      + [SHOW DYNAMIC PARTITION TABLES](./sql-reference/sql-statements/data-manipulation/SHOW%20DYNAMIC%20PARTITION%20TABLES.md)
      + [SHOW EXPORT](./sql-reference/sql-statements/data-manipulation/SHOW%20EXPORT.md)
      + [SHOW LOAD](./sql-reference/sql-statements/data-manipulation/SHOW%20LOAD.md)
      + [SHOW PARTITIONS](./sql-reference/sql-statements/data-manipulation/SHOW%20PARTITIONS.md)
      + [SHOW PROPERTY](./sql-reference/sql-statements/data-manipulation/SHOW%20PROPERTY.md)
      + [SHOW REPOSITORIES](./sql-reference/sql-statements/data-manipulation/SHOW%20REPOSITORIES.md)
      + [SHOW RESTORE](./sql-reference/sql-statements/data-manipulation/SHOW%20RESTORE.md)
      + [SHOW ROUTINE LOAD](./sql-reference/sql-statements/data-manipulation/SHOW%20ROUTINE%20LOAD.md)
      + [SHOW ROUTINE LOAD TASK](./sql-reference/sql-statements/data-manipulation/SHOW%20ROUTINE%20LOAD%20TASK.md)
      + [SHOW SNAPSHOT](./sql-reference/sql-statements/data-manipulation/SHOW%20SNAPSHOT.md)
      + [SHOW TABLES](./sql-reference/sql-statements/data-manipulation/SHOW%20TABLES.md)
      + [SHOW TABLET](./sql-reference/sql-statements/data-manipulation/SHOW%20TABLET.md)
      + [SHOW TRANSACTION](./sql-reference/sql-statements/data-manipulation/SHOW%20TRANSACTION.md)
      + [SPARK LOAD](./sql-reference/sql-statements/data-manipulation/SPARK%20LOAD.md)
      + [STOP ROUTINE LOAD](./sql-reference/sql-statements/data-manipulation/STOP%20ROUTINE%20LOAD.md)
      + [STREAM LOAD](./sql-reference/sql-statements/data-manipulation/STREAM%20LOAD.md)
    + Data Types
      + Numeric
        + [BIGINT](./sql-reference/sql-statements/data-types/BIGINT.md)
        + [LARGEINT](./sql-reference/sql-statements/data-types/LARGEINT.md)
        + [SMALLINT](./sql-reference/sql-statements/data-types/SMALLINT.md)
        + [TINYINT](./sql-reference/sql-statements/data-types/TINYINT.md)
        + [BOOLEAN](./sql-reference/sql-statements/data-types/BOOLEAN.md)
        + [DECIMAL](./sql-reference/sql-statements/data-types/DECIMAL.md)
        + [DOUBLE](./sql-reference/sql-statements/data-types/DOUBLE.md)
        + [FLOAT](./sql-reference/sql-statements/data-types/FLOAT.md)
        + [INT](./sql-reference/sql-statements/data-types/INT.md)
      + String
        + [CHAR](./sql-reference/sql-statements/data-types/CHAR.md)
        + [VARCHAR](./sql-reference/sql-statements/data-types/VARCHAR.md)
        + [STRING](./sql-reference/sql-statements/data-types/STRING.md)
      + Date
        + [DATE](./sql-reference/sql-statements/data-types/DATE.md)
        + [DATETIME](./sql-reference/sql-statements/data-types/DATETIME.md)
      + Others
        + [HLL](./sql-reference/sql-statements/data-types/HLL.md)
        + [BITMAP](./sql-reference/sql-statements/data-types/BITMAP.md)
        + [JSON](./sql-reference/sql-statements/data-types/JSON.md)
    + Auxiliary Commands
      + [DESCRIBE](./sql-reference/sql-statements/Utility/DESCRIBE.md)
  + Function Reference
    + Date Functions
      + [convert_tz](./sql-reference/sql-functions/date-time-functions/convert_tz.md)
      + [curdate](./sql-reference/sql-functions/date-time-functions/curdate.md)
      + [current_timestamp](./sql-reference/sql-functions/date-time-functions/current_timestamp.md)
      + [curtime](./sql-reference/sql-functions/date-time-functions/curtime.md)
      + [datediff](./sql-reference/sql-functions/date-time-functions/datediff.md)
      + [date_add](./sql-reference/sql-functions/date-time-functions/date_add.md)
      + [date_format](./sql-reference/sql-functions/date-time-functions/date_format.md)
      + [date_sub](./sql-reference/sql-functions/date-time-functions/date_sub.md)
      + [date_trunc](./sql-reference/sql-functions/date-time-functions/date_trunc.md)
      + [day](./sql-reference/sql-functions/date-time-functions/day.md)
      + [dayname](./sql-reference/sql-functions/date-time-functions/dayname.md)
      + [dayofmonth](./sql-reference/sql-functions/date-time-functions/dayofmonth.md)
      + [dayofweek](./sql-reference/sql-functions/date-time-functions/dayofweek.md)
      + [dayofyear](./sql-reference/sql-functions/date-time-functions/dayofyear.md)
      + [from_days](./sql-reference/sql-functions/date-time-functions/from_days.md)
      + [from_unixtime](./sql-reference/sql-functions/date-time-functions/from_unixtime.md)
      + [hour](./sql-reference/sql-functions/date-time-functions/hour.md)
      + [minute](./sql-reference/sql-functions/date-time-functions/minute.md)
      + [month](./sql-reference/sql-functions/date-time-functions/month.md)
      + [monthname](./sql-reference/sql-functions/date-time-functions/monthname.md)
      + [now](./sql-reference/sql-functions/date-time-functions/now.md)
      + [second](./sql-reference/sql-functions/date-time-functions/second.md)
      + [str_to_date](./sql-reference/sql-functions/date-time-functions/str_to_date.md)
      + [timediff](./sql-reference/sql-functions/date-time-functions/timediff.md)
      + [timestampadd](./sql-reference/sql-functions/date-time-functions/timestampadd.md)
      + [timestampdiff](./sql-reference/sql-functions/date-time-functions/timestampdiff.md)
      + [to_date](./sql-reference/sql-functions/date-time-functions/to_date.md)
      + [to_days](./sql-reference/sql-functions/date-time-functions/to_days.md)
      + [unix_timestamp](./sql-reference/sql-functions/date-time-functions/unix_timestamp.md)
      + [utc_timestamp](./sql-reference/sql-functions/date-time-functions/utc_timestamp.md)
      + [weekofyear](./sql-reference/sql-functions/date-time-functions/weekofyear.md)
      + [year](./sql-reference/sql-functions/date-time-functions/year.md)
    + Geographic Functions
      + [st_astext](./sql-reference/sql-functions/spatial-functions/st_astext.md)
      + [st_circle](./sql-reference/sql-functions/spatial-functions/st_circle.md)
      + [st_contains](./sql-reference/sql-functions/spatial-functions/st_contains.md)
      + [st_distance_sphere](./sql-reference/sql-functions/spatial-functions/st_distance_sphere.md)
      + [st_geometryfromtext](./sql-reference/sql-functions/spatial-functions/st_geometryfromtext.md)
      + [st_linefromtext](./sql-reference/sql-functions/spatial-functions/st_linefromtext.md)
      + [st_point](./sql-reference/sql-functions/spatial-functions/st_point.md)
      + [st_polygon](./sql-reference/sql-functions/spatial-functions/st_polygon.md)
      + [st_x](./sql-reference/sql-functions/spatial-functions/st_x.md)
      + [st_y](./sql-reference/sql-functions/spatial-functions/st_y.md)
    + String Functions
      + [append_trailing_char_if_absent](./sql-reference/sql-functions/string-functions/append_trailing_char_if_absent.md)
      + [ascii](./sql-reference/sql-functions/string-functions/ascii.md)
      + [char_length](./sql-reference/sql-functions/string-functions/char_length.md)
      + [concat](./sql-reference/sql-functions/string-functions/concat.md)
      + [concat_ws](./sql-reference/sql-functions/string-functions/concat_ws.md)
      + [ends_with](./sql-reference/sql-functions/string-functions/ends_with.md)
      + [find_in_set](./sql-reference/sql-functions/string-functions/find_in_set.md)
      + [get_json_double](./sql-reference/sql-functions/string-functions/get_json_double.md)
      + [get_json_int](./sql-reference/sql-functions/string-functions/get_json_int.md)
      + [get_json_string](./sql-reference/sql-functions/string-functions/get_json_string.md)
      + [group_concat](./sql-reference/sql-functions/string-functions/group_concat.md)
      + [instr](./sql-reference/sql-functions/string-functions/instr.md)
      + [lcase](./sql-reference/sql-functions/string-functions/lcase.md)
      + [left](./sql-reference/sql-functions/string-functions/left.md)
      + [length](./sql-reference/sql-functions/string-functions/length.md)
      + [locate](./sql-reference/sql-functions/string-functions/locate.md)
      + [lower](./sql-reference/sql-functions/string-functions/lower.md)
      + [lpad](./sql-reference/sql-functions/string-functions/lpad.md)
      + [ltrim](./sql-reference/sql-functions/string-functions/ltrim.md)
      + [money_format](./sql-reference/sql-functions/string-functions/money_format.md)
      + [null_or_empty](./sql-reference/sql-functions/string-functions/null_or_empty.md)
      + [regexp_extract](./sql-reference/sql-functions/string-functions/regexp_extract.md)
      + [regexp_replace](./sql-reference/sql-functions/string-functions/regexp_replace.md)
      + [repeat](./sql-reference/sql-functions/string-functions/repeat.md)
      + [reverse](./sql-reference/sql-functions/string-functions/reverse.md)
      + [right](./sql-reference/sql-functions/string-functions/right.md)
      + [rpad](./sql-reference/sql-functions/string-functions/rpad.md)
      + [split](./sql-reference/sql-functions/string-functions/split.md)
      + [split_part](./sql-reference/sql-functions/string-functions/split_part.md)
      + [starts_with](./sql-reference/sql-functions/string-functions/starts_with.md)
      + [strleft](./sql-reference/sql-functions/string-functions/strleft.md)
      + [strright](./sql-reference/sql-functions/string-functions/strright.md)
    + JSON Functions
      + [Overview of JSON functions and operators](./sql-reference/sql-functions/json-functions/overview-of-json-functions-and-operators.md)
      + JSON constructor functions
        + [json_array](./sql-reference/sql-functions/json-functions/json-constructor-functions/json_array.md)
        + [json_object](./sql-reference/sql-functions/json-functions/json-constructor-functions/json_object.md)
        + [parse_json](./sql-reference/sql-functions/json-functions/json-constructor-functions/parse_json.md)
      + JSON query and processing functions
        + [Arrow function](./sql-reference/sql-functions/json-functions/json-query-and-processing-functions/arrow-function.md)
        + [json_query](./sql-reference/sql-functions/json-functions/json-query-and-processing-functions/json_query.md)
        + [json_exists](./sql-reference/sql-functions/json-functions/json-query-and-processing-functions/json_exists.md)
        + [json_each](./sql-reference/sql-functions/json-functions/json-query-and-processing-functions/json_each.md)
        + [cast](./sql-reference/sql-functions/json-functions/json-query-and-processing-functions/cast.md)
      + [JSON operators](./sql-reference/sql-functions/json-functions/json-operators.md)
    + Aggregate Functions
      + [any_value](./sql-reference/sql-functions/aggregate-functions/any_value.md)
      + [approx_count_distinct](./sql-reference/sql-functions/aggregate-functions/approx_count_distinct.md)
      + [avg](./sql-reference/sql-functions/aggregate-functions/avg.md)
      + [bitmap](./sql-reference/sql-functions/aggregate-functions/bitmap.md)
      + [count](./sql-reference/sql-functions/aggregate-functions/count.md)
      + [hll_union_agg](./sql-reference/sql-functions/aggregate-functions/hll_union_agg.md)
      + [max](./sql-reference/sql-functions/aggregate-functions/max.md)
      + [min](./sql-reference/sql-functions/aggregate-functions/min.md)
      + [percentile_approx](./sql-reference/sql-functions/aggregate-functions/percentile_approx.md)
      + [retention](./sql-reference/sql-functions/aggregate-functions/retention.md)
      + [stddev](./sql-reference/sql-functions/aggregate-functions/stddev.md)
      + [stddev_samp](./sql-reference/sql-functions/aggregate-functions/stddev_samp.md)
      + [sum](./sql-reference/sql-functions/aggregate-functions/sum.md)
      + [variance](./sql-reference/sql-functions/aggregate-functions/variance.md)
      + [square](./sql-reference/sql-functions/aggregate-functions/square.md)
      + [var_samp](./sql-reference/sql-functions/aggregate-functions/var_samp.md)
    + Bitmap Functions
      + [bitmap_and](./sql-reference/sql-functions/bitmap-functions/bitmap_and.md)
      + [bitmap_andnot](./sql-reference/sql-functions/bitmap-functions/bitmap_andnot.md)
      + [bitmap_contains](./sql-reference/sql-functions/bitmap-functions/bitmap_contains.md)
      + [bitmap_empty](./sql-reference/sql-functions/bitmap-functions/bitmap_empty.md)
      + [bitmap_from_string](./sql-reference/sql-functions/bitmap-functions/bitmap_from_string.md)
      + [bitmap_has_any](./sql-reference/sql-functions/bitmap-functions/bitmap_has_any.md)
      + [bitmap_hash](./sql-reference/sql-functions/bitmap-functions/bitmap_hash.md)
      + [bitmap_intersect](./sql-reference/sql-functions/bitmap-functions/bitmap_intersect.md)
      + [bitmap_max](./sql-reference/sql-functions/bitmap-functions/bitmap_max.md)
      + [bitmap_min](./sql-reference/sql-functions/bitmap-functions/bitmap_min.md)
      + [bitmap_or](./sql-reference/sql-functions/bitmap-functions/bitmap_or.md)
      + [bitmap_remove](./sql-reference/sql-functions/bitmap-functions/bitmap_remove.md)
      + [bitmap_to_string](./sql-reference/sql-functions/bitmap-functions/bitmap_to_string.md)
      + [bitmap_union](./sql-reference/sql-functions/bitmap-functions/bitmap_union.md)
      + [bitmap_xor](./sql-reference/sql-functions/bitmap-functions/bitmap_xor.md)
      + [to_bitmap](./sql-reference/sql-functions/bitmap-functions/to_bitmap.md)
    + Array Functions
      + [array_agg](./sql-reference/sql-functions/array-functions/array_agg.md)
      + [array_append](./sql-reference/sql-functions/array-functions/array_append.md)
      + [array_avg](./sql-reference/sql-functions/array-functions/array_avg.md)
      + [array_concat](./sql-reference/sql-functions/array-functions/array_concat.md)
      + [array_contains](./sql-reference/sql-functions/array-functions/array_contains.md)
      + [array_difference](./sql-reference/sql-functions/array-functions/array_difference.md)
      + [array_distinct](./sql-reference/sql-functions/array-functions/array_distinct.md)
      + [array_intersect](./sql-reference/sql-functions/array-functions/array_intersect.md)
      + [array_join](./sql-reference/sql-functions/array-functions/array_join.md)
      + [array_length](./sql-reference/sql-functions/array-functions/array_length.md)
      + [array_max](./sql-reference/sql-functions/array-functions/array_max.md)
      + [array_min](./sql-reference/sql-functions/array-functions/array_min.md)
      + [array_position](./sql-reference/sql-functions/array-functions/array_position.md)
      + [array_remove](./sql-reference/sql-functions/array-functions/array_remove.md)
      + [array_slice](./sql-reference/sql-functions/array-functions/array_slice.md)
      + [array_sort](./sql-reference/sql-functions/array-functions/array_sort.md)
      + [array_sum](./sql-reference/sql-functions/array-functions/array_sum.md)
      + [arrays_overlap](./sql-reference/sql-functions/array-functions/arrays_overlap.md)
      + [reverse](./sql-reference/sql-functions/array-functions/reverse.md)
    + [cast function](./sql-reference/sql-functions/cast.md)
    + [hash function](./sql-reference/sql-functions/hash-functions/murmur_hash3_32.md)
    + Cryptographic Functions
      + [md5](./sql-reference/sql-functions/crytographic-functions/md5.md)
      + [sha2](./sql-reference/sql-functions/crytographic-functions/sha2.md)
    + Math Functions
      + [RAND](./sql-reference/sql-functions/math-functions/rand.md)
      + [square](./sql-reference/sql-functions/math-functions/square.md)
    + Utility Functions
      + [uuid](./sql-reference/sql-functions/utility-functions/uuid.md)
      + [uuid_numeric](./sql-reference/sql-functions/utility-functions/uuid_numeric.md)
  + [System variables](./reference/System_variable.md)
  + [Error code](./reference/Error_code.md)
  + [System limits](./reference/System_limit.md)
+ Administration
  + [Build in Docker](./administration/Build_in_docker.md)
  + [Manage a cluster](./administration/Cluster_administration.md)
  + [Configuration](./administration/Configuration.md)
  + [Scale In and Out](./administration/Scale_up_down.md)
  + [Backup and Restore](./administration/Backup_and_restore.md)
  + Data Recovery
    + [Data Recovery](./administration/Data_recovery.md)
    + [Restore FEs](./administration/Metadata_recovery.md)
  + [Replica](./administration/Replica.md)
  + [Query Planning](./administration/Query_planning.md)
  + [Profiling](./administration/Profiling.md)
  + [Monitor and Alert](./administration/Monitor_and_Alert.md)
  + [User Privilege](./administration/User_privilege.md)
  + [Authentication](./administration/Authentication.md)
  + Resource Management
    + [Resource Group](./administration/resource_group.md)
    + [Query Management](./administration/Query_management.md)
    + [Blacklist](./administration/Blacklist.md)
    + [Memory Management](./administration/Memory_management.md)
    + [Load Balance](./administration/Load_balance.md)
+ FAQ
  + [Deploy](./faq/Deploy_faq.md)
  + Data Migration
    + Data Ingestion
      + [Data Ingestion](./faq/loading/Loading_faq.md)
      + [Stream Load](./faq/loading/Stream_load_faq.md)
      + [Routine Load](./faq/loading/Routine_load_faq.md)
      + [Broker Load](./faq/loading/Broker_load_faq.md)
      + [Insert Into](./faq/loading/Insert_into_faq.md)
      + [Flink connector](./faq/loading/Flink_connector_faq.md)
      + [DataX](./faq/loading/DataX_faq.md)
    + [Data Export](./faq/Exporting_faq.md)
  + [SQL](./faq/Sql_faq.md)
  + [Other FAQs](./faq/Others.md)
+ Benchmark
  + [SSB Benchmark](./benchmarking/SSB_Benchmarking.md)
  + [TPC-H Benchmark](./benchmarking/TPC-H_Benchmarking.md)
+ Release Notes
  + [v2.2](./release_notes/release-2.2.md)
  + [v2.1](./release_notes/release-2.1.md)
  + [v2.0](./release_notes/release-2.0.md)
  + [v1.19](./release_notes/release-1.19.md)
